"""
    Copyright 2021 Hendrik Weimer

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""

print("ising: Begin loading modules")

from js import window, document, pixup, pixdown
from math import *
import random
from time import time

print("ising: End loading modules")

lstatus = document.getElementById("status")

def push_queue(func, str):
    lstatus.innerHTML = str
    window.setTimeout(func, 10)

globals = dict()

globals['beta'] = 0.1
globals['Jx'] = 1
globals['Jy'] = 1
globals['Nx'] = 400
globals['Ny'] = 200

globals['S'] = [[1] * globals['Ny'] for i in range(globals['Nx'])]

globals['firstrun'] = 1

def replot_canvas():

    beginning = time()

    canvas = document.getElementById("canvas")
    ctx = canvas.getContext("2d")

    Nx = globals['Nx']
    Ny = globals['Ny']

    Jx = globals['Jx']
    Jy = globals['Jy']

    while time()-beginning < 0.04:

        if document.getElementById("cluster").checked:
            # Wolff algorithm [doi:10.1007/3-540-35273-2]

            # Choosing which spin to start with

            i = random.randint(0, Nx-1)
            j = random.randint(0, Ny-1)

            px = 1-exp(-2*globals['beta']*Jx)
            py = 1-exp(-2*globals['beta']*Jy)

            stack = [(i, j)]
            sign = globals['S'][i][j]
            globals['S'][i][j] *= -1

            if sign == 1:
                ctx.putImageData(pixdown, i, j)
            else:
                ctx.putImageData(pixup, i, j)
            
            while len(stack):

                i, j = stack.pop()

                for pos in [(i, (j+1)%Ny), (i, (j-1)%Ny)]: # north and south
                    if (sign == globals['S'][pos[0]][pos[1]])\
                       and (py > random.random()):
                        stack.append(pos)
                        globals['S'][pos[0]][pos[1]] *= -1

                        if sign == 1:
                            ctx.putImageData(pixdown, pos[0], pos[1])
                        else:
                            ctx.putImageData(pixup, pos[0], pos[1])

                for pos in [((i+1)%Nx, j), ((i-1)%Nx, j)]: # east and west
                    if (sign == globals['S'][pos[0]][pos[1]])\
                       and (px > random.random()):
                        stack.append(pos)
                        globals['S'][pos[0]][pos[1]] *= -1

                        if sign == 1:
                            ctx.putImageData(pixdown, pos[0], pos[1])
                        else:
                            ctx.putImageData(pixup, pos[0], pos[1])


        else:
        
            # Choosing which spin to flip

            i = random.randint(0, Nx-1)
            j = random.randint(0, Ny-1)

            Sc = globals['S'][i][j] # (flipped) center
            SN = globals['S'][i][(j+1)%Ny] # north
            SE = globals['S'][(i+1)%Nx][j] # east
            SS = globals['S'][i][(j-1)%Ny] # south
            SW = globals['S'][(i-1)%Nx][j] # west

            # change in energy

            DE = 2*Sc*(Jx*(SE+SW) + Jy*(SN+SS))

            # Monte-Carlo step

            if exp(-globals['beta']*DE) > random.random():
                globals['S'][i][j] *= -1

                if Sc == 1:
                    ctx.putImageData(pixdown, i, j)
                else:
                    ctx.putImageData(pixup, i, j)

#    print(time()-beginning)


def change_inputs(params):

    print("ising: entering change_inputs")
    
    window.clearInterval(globals['timer'])

    fail = 0

    ibeta = document.getElementById("beta")
    lbeta = document.getElementById("lbeta")

    iJx = document.getElementById("Jx")
    lJx = document.getElementById("lJx")

    iJy = document.getElementById("Jy")
    lJy = document.getElementById("lJy")


    try:
        globals['beta'] = eval(ibeta.value)
        if not isfinite(globals['beta']):
            raise ValueError
        lbeta.innerHTML = ''
    except:
        lbeta.innerHTML = "Input error"
        lstatus.innerHTML = ""
        fail = 1

    try:
        globals['Jx'] = eval(iJx.value)
        if not isfinite(globals['Jx']):
            raise ValueError
        lJx.innerHTML = ''
    except:
        lJx.innerHTML = "Input error"
        lstatus.innerHTML = ""
        fail = 1

    try:
        globals['Jy'] = eval(iJy.value)
        if not isfinite(globals['Jy']):
            raise ValueError
        lJy.innerHTML = ''
    except:
        lJy.innerHTML = "Input error"
        lstatus.innerHTML = ""
        fail = 1


    if fail == 0:

        print("ising: exiting change_inputs")
        globals['timer'] = window.setInterval(replot_canvas, 40)




def set_main():

    print("idealgas: entering set_main")

    global cwidth, cheight

    canvas = document.getElementById("canvas")

    parentwidth = document.getElementById("parent").clientWidth
    cwidth = parentwidth
    cheight = int(cwidth*0.75)
    
    canvas.setAttribute("width", globals['Nx'])
    canvas.setAttribute("height", globals['Ny'])

    ctx = canvas.getContext("2d")

#    globals['pixup'] = ctx.createImageData(1,1)
#    globals['pixup'].data[0] = 0 # R
#    globals['pixup'].data[1] = 80 # G
#    globals['pixup'].data[2] = 155 # B
#    globals['pixup'].data[3] = 255 # A

#    globals['pixdown'] = ctx.createImageData(1,1)
#    globals['pixdown'].data[0] = 230 # R
#    globals['pixdown'].data[1] = 159 # G
#    globals['pixdown'].data[2] = 0 # B
#    globals['pixdown'].data[3] = 255 # A

#    print("0", globals['pixdown'].data[0])
#    print("1", globals['pixdown'].data[1])
#    print("2", globals['pixdown'].data[2])
#    print("3", globals['pixdown'].data[3])

    ctx.fillStyle = "#00509B"
    ctx.fillRect(0, 0, globals['Nx'], globals['Ny'])

    print("setting timer")

    globals['timer'] = window.setInterval(replot_canvas, 40)

    if globals['firstrun'] == 1:

        for i in ["beta", "Jx", "Jy"]:
            ii = document.getElementById(i)
            ii.addEventListener("change", change_inputs)

        globals['firstrun'] = 0

    lstatus.innerHTML = ""

    print("ising: exiting set_main")

push_queue(set_main, "Setting up main window...")

